/****************************************/
/* Logging facility (2nd version)	*/
/****************************************/
/* Start      : 14/04/2004		*/
/* Version    : 1.00			*/
/* Last Update: 14/04/2004		*/
/****************************************/

#include "log2.h"

#include <v/misc/keytable2.cc>

// Syslog levels mapping
int	LOGGER2::syslogmap[8]=
		{LOG_EMERG, LOG_ALERT, LOG_CRIT, LOG_ERR, LOG_WARNING,
			LOG_NOTICE, LOG_INFO, LOG_DEBUG};

// Instances that use syslog
// This was syslog may be closed by the last class
int	LOGGER2::syslog_count=0;

LOGGER2::LOGGER2()
{
	minlevel=1000;
	facility=LogNone;
	f=NULL;
	logtype=LOGTYPE_TIME | LOGTYPE_ENTER | LOGTYPE_CLASS;
}

LOGGER2::~LOGGER2()
{
	CloseLog();
}

void LOGGER2::ddd(LOGGER2 &d)
{
	minlevel=d.minlevel;
	loglevels=d.loglevels;
	facility=d.facility;
	logtype=d.logtype;

	if (facility==LogSyslog)
	{
		syslog_count++;
	} else if (facility==LogFile)
	{
		SetFile(d.fn.p);
	} else if (facility==LogStdout)
	{
		SetStdout();
	} else if (facility==LogStderr)
	{
		SetStderr();
	}
}

LOGGER2::LOGGER2(LOGGER2 &d)
{
	ddd(d);
}

LOGGER2 LOGGER2::operator=(LOGGER2 &d)
{
	if (this==&d)
		return(*this);

	CloseLog();	// first close old log

	ddd(d);
	return(*this);
}

/*
 * Try to open file for appending.
 * Create it if it doesn't exist
 *
 * Return:
 * 	1: OK
 * 	0: Error
 */
int	LOGGER2::OpenFile(const char *fn)
{
	if (!strcmp(this->fn.p, fn))	// same file
		return(1);

	CloseLog();
	
	f=fopen(fn, "a+t");
	if (f==NULL)
		return(0);

	this->f=f;
	this->fn=fn;

	facility=LogFile;

	return(1);
}

void	LOGGER2::CloseFile()
{
	if (f!=NULL)
	{
		if (f!=stdout && f!=stderr)
			fclose(f);

		f=NULL;
		fn="";
	}
}

void	LOGGER2::CloseLog()
{
	if (facility==LogSyslog)
	{
		syslog_count--;
		if (syslog_count==0)
			closelog();	// close syslog
	} else
	{
		CloseFile();	// close possible open file
	}
	facility=LogNone;
}

void	LOGGER2::SetLevel(int n)
{
	minlevel=n;
}

void	LOGGER2::SetLevel(const char *p, int on)
{
	LOGLEVEL	*pll;

	pll=loglevels[p];
	if (!pll) return;

	pll->on=on;
}

void	LOGGER2::SetLevel(const LONGLINE &ll, int on)
{
	SetLevel(ll.p, on);
}

void	LOGGER2::AddLevel(const char *name, int on)
{
	LOGLEVEL	*pll;

	pll=loglevels[name];
	if (pll==NULL)
	{
		loglevels.Add(name, LOGLEVEL());
		pll=loglevels[name];
	}

	pll->name=name;
	pll->on=on;
}

void	LOGGER2::AddLevel(const LONGLINE &ll, int on)
{
	AddLevel(ll.p, on);
}

void	LOGGER2::DelLevel(const char *name)
{
	loglevels.Del(name);
}

void	LOGGER2::DelLevel(const LONGLINE &ll)
{
	DelLevel(ll.p);
}

/*
 * Set logfile. Create it if it doesn't exist
 *
 * Return:
 * 	1: OK
 * 	0: Error
 */
int	LOGGER2::SetFile(const char *fn)
{
	return(OpenFile(fn));
}

void	LOGGER2::SetStdout()
{
	CloseLog();
	facility=LogStdout;
	this->fn="STDOUT";
	this->f=stdout;
}

void	LOGGER2::SetStderr()
{
	CloseLog();
	facility=LogStderr;
	this->fn="STDERR";
	this->f=stderr;
}

/*
 * ident is the ident string that will be prepended to syslog messages
 * facility is the syslog facility that we will log to.
 */
void	LOGGER2::SetSyslog(const char *ident, int facility)
{
	if (this->facility==LogSyslog)
		return;

	CloseLog();
	this->facility=LogSyslog;
	
	if (syslog_count==0)
		openlog(ident, LOG_PID, facility);

	syslog_count++;
}

void	LOGGER2::SetLogType(int n)
{
	logtype=n;
}

/*
 * Log to a file.
 *
 * Used by plog()
 */
void	LOGGER2::plogFile(const char *prefix, const char *format, va_list l)
{
	int	n;
	char	ch1, ch2;

	if (logtype & LOGTYPE_TIME)
	{
		fputs(vtime.str(), this->f);
		fputs(": ", this->f);
	}

	if (logtype & LOGTYPE_CLASS)
	{
		fputs(prefix, this->f);
		fputs(": ", this->f);
	}

	vfprintf(this->f, format, l);

	if (logtype & LOGTYPE_ENTER)
	{
		n=strlen(format);
		ch1=format[n-1];
		ch2=format[n-2];
		if (ch1!='\n' || (ch1=='\r' && ch2!='\n'))
		{
			fputc('\n', this->f);
		}
	}

	fflush(this->f);
}

/*
 * Log to file or syslog
 */
void	LOGGER2::plog(int level, const char *format, va_list l)
{
	int	lev2;
	char	tmp[10];

	if (facility==LogNone)
		return;

	if (level>minlevel)
		return;

	if (facility==LogSyslog)
	{
		if (level<0)
			level=0;
		else if (level>7)
			level=7;

		lev2=syslogmap[level];

		vsyslog(lev2, format, l);
	} else
	{
		sprintf(tmp, "L%d", level%10000);	// Just to be sure of overflows

		plogFile(tmp, format, l);
	}
}

void	LOGGER2::plog(const char *levelstr, const char *format, va_list l)
{
	if (facility==LogNone)
		return;

	if (!checklevel(levelstr))
		return;

	if (facility==LogSyslog)
	{
		// TODO: Customizable syslog levels for string loglevels
		vsyslog(LOG_INFO, format, l);
	} else
	{
		plogFile(levelstr, format, l);
	}
}

/*
 * Check if a string level should be logged
 *
 * Return:
 * 	1: Yes
 * 	0: No
 */
int	LOGGER2::checklevel(const char *level)
{
	LOGLEVEL	*pll;

	pll=loglevels[level];

	if (!pll) return(0);

	if (pll->on) return(1);

	return(0);
}

void	LOGGER2::Log(int level, const char *format, ...)
{
	va_list l;
	
	va_start(l, format);
	plog(level, format, l);
	va_end(l);
}

void	LOGGER2::log(int level, const char *format, ...)
{
	va_list l;
	
	va_start(l, format);
	plog(level, format, l);
	va_end(l);
}

void	LOGGER2::vlog(int level, const char *format, va_list l)
{
	plog(level, format, l);
}

void	LOGGER2::vLog(int level, const char *format, va_list l)
{
	plog(level, format, l);
}

void	LOGGER2::Log(const char *level, const char *format, ...)
{
	va_list l;
	
	va_start(l, format);
	plog(level, format, l);
	va_end(l);
}

void	LOGGER2::log(const char *level, const char *format, ...)
{
	va_list l;
	
	va_start(l, format);
	plog(level, format, l);
	va_end(l);
}

void	LOGGER2::vlog(const char *level, const char *format, va_list l)
{
	plog(level, format, l);
}

void	LOGGER2::vLog(const char *level, const char *format, va_list l)
{
	plog(level, format, l);
}

